﻿namespace DataLoader
{
    using System;
    using System.Collections.Generic;
    using System.Linq;
    using System.Text;
    using DataLoader.Common.Model;
    using System.Text.RegularExpressions;
    using System.IO;

    internal class ResponseParser
    {
        internal static List<string> ParseCreatePatientResponse(string rawData, Patient pnt)
        {
            List<string> rlist = new List<string>();
            if (rawData == null)
            {
                rlist.Add("error");
                rlist.Add("No reply from VistA.");
                return rlist;
            }
            string results = rawData.ToString();
            string[] lines;
            try
            {
                lines = Regex.Split(results, "\r\n");
            }
            catch
            {
                rlist.Add("error");
                rlist.Add("No expected reply from VistA.");
                return rlist;
            }
            pnt.Clear();
            if ((lines != null) && (lines.Length > 0))
            {
                if (lines[0] == "1")
                {
                    string[] fields = lines[1].Split('^');
                    if ((fields != null) && (fields.Length > 0))
                    {
                        pnt.PatientDFN = fields[0];
                        pnt.SSN = fields[1];
                        pnt.PatientName = fields[2];
                        pnt.CaseName = Case.CaseName;
                        rlist.Add("success");
                        rlist.Add(pnt.PatientName);
                        rlist.Add(pnt.PatientDFN);
                        rlist.Add(pnt.SSN);
                    }
                }
                else
                {
                    string[] fields = lines[0].Split('^');
                    if ((fields != null) && (fields.Length > 0))
                    {
                        if (fields[0] == "-1")
                        {
                            rlist.Add("error");
                            if (fields[1] != null)
                            {
                                rlist.Add(fields[1].ToString());
                            }
                        }
                    }
                }
            } 
            return rlist;
        }

        internal static List<string> ParseCreateApptResponse(string rawData)
        {
            List<string> rlist = new List<string>();
            if (rawData == null)
            {
                rlist.Add("error");
                rlist.Add("VistA error!");
                return rlist;
            }
            string results = rawData.ToString();
            string[] lines = Regex.Split(results, "\r\n");
            if ((lines != null) && (lines.Length > 0))
            {
                if (lines[0] == "1")
                {
                    string[] fields = lines[1].Split('^');
                    if ((fields != null) && (fields.Length > 0))
                    {
                        rlist.Add("success");
                        //rlist.Add(fields[1].ToString());
                    }
                }
                else
                {
                    string[] fields = lines[0].Split('^');
                    if ((fields != null) && (fields.Length > 0))
                    {
                        if (fields[0] == "-1")
                        {
                            rlist.Add("error");
                            if (fields[1] != null)
                            {
                                rlist.Add(fields[1].ToString());
                            }
                        }
                    }
                }
            }
            return rlist;
        }

        internal static List<string> ParseCreateProbResponse(string rawData)
        {
            List<string> rlist = new List<string>();
            if (rawData == null)
            {
                rlist.Add("error");
                rlist.Add("VistA error!");
                return rlist;
            }
            string results = rawData.ToString();
            string[] lines = Regex.Split(results, "\r\n");
            if ((lines != null) && (lines.Length > 0))
            {
                if (lines[0] == "1")
                {
                    string[] fields = lines[1].Split('^');
                    if ((fields != null) && (fields.Length > 0))
                    {
                        rlist.Add("success");
                        //rlist.Add(fields[1].ToString());
                    }
                }
                else
                {
                    string[] fields = lines[0].Split('^');
                    if ((fields != null) && (fields.Length > 0))
                    {
                        if (fields[0] == "-1")
                        {
                            rlist.Add("error");
                            if (fields[1] != null)
                            {
                                rlist.Add(fields[1].ToString());
                            }
                        }
                    }
                }
            }
            return rlist;
        }

        internal static List<string> ParseCreateAllergyResponse(string rawData)
        {
            List<string> rlist = new List<string>();
            if (rawData == null)
            {
                rlist.Add("error");
                rlist.Add("VistA error!");
                return rlist;
            }
            string results = rawData.ToString();
            string[] lines = Regex.Split(results, "\r\n");
            if ((lines != null) && (lines.Length > 0))
            {
                if (lines[0] == "1")
                {
                    string[] fields = lines[1].Split('^');
                    if ((fields != null) && (fields.Length > 0))
                    {
                        rlist.Add("success");
                        //rlist.Add(fields[1].ToString());
                    }
                }
                else
                {
                    string[] fields = lines[0].Split('^');
                    if ((fields != null) && (fields.Length > 0))
                    {
                        if (fields[0] == "-1")
                        {
                            rlist.Add("error");
                            if (fields[1] != null)
                            {
                                rlist.Add(fields[1].ToString());
                            }
                        }
                    }
                }
            }
            return rlist;
        }

        internal static List<string> ParseCreateVitalResponse(string rawData)
        {
            List<string> rlist = new List<string>();
            if (rawData == null)
            {
                rlist.Add("error");
                rlist.Add("VistA error!");
                return rlist;
            }
            string results = rawData.ToString();
            string[] lines = Regex.Split(results, "\r\n");
            if ((lines != null) && (lines.Length > 0))
            {
                if (lines[0] == "1")
                {
                    string[] fields = lines[1].Split('^');
                    if ((fields != null) && (fields.Length > 0))
                    {
                        rlist.Add("success");
                        //rlist.Add(fields[1].ToString());
                    }
                }
                else
                {
                    string[] fields = lines[0].Split('^');
                    if ((fields != null) && (fields.Length > 0))
                    {
                        if (fields[0] == "-1")
                        {
                            rlist.Add("error");
                            if (fields[1] != null)
                            {
                                rlist.Add(fields[1].ToString());
                            }
                        }
                    }
                }
            }
            return rlist;
        }

        internal static List<string> ParseCreateLabResponse(string rawData)
        {
            List<string> rlist = new List<string>();
            if (rawData == null)
            {
                //There is an issue with GT.M where the connection is dropping, despite no error on the LABS
                rlist.Add("confirm");
                //rlist.Add("no");
                return rlist;
            }
            string results = rawData.ToString();
            string[] lines;
            try
            {
                lines = Regex.Split(results, "\r\n");
            }
            catch
            {
                lines = null;
                rlist.Add("confirm");
            }
            if ((lines != null) && (lines.Length > 0))
            {
                if (lines[0] == "1")
                {
                    string[] fields = lines[1].Split('^');
                    if ((fields != null) && (fields.Length > 0))
                    {
                        rlist.Add("success");
                        //rlist.Add(fields[1].ToString());
                    }
                }
                else
                {
                    string[] fields = lines[0].Split('^');
                    if ((fields != null) && (fields.Length > 0))
                    {
                        if (fields[0] == "-1")
                        {
                            rlist.Add("error");
                            if (fields[1] != null)
                            {
                                rlist.Add(fields[1].ToString());
                            }
                        }
                    }
                }
            }
            return rlist;
        }

        internal static List<string> ParseCreateNoteResponse(string rawData)
        {
            List<string> rlist = new List<string>();
            if (rawData == null)
            {
                rlist.Add("error");
                rlist.Add("VistA error!");
                return rlist;
            }
            string results = rawData.ToString();
            string[] lines = Regex.Split(results, "\r\n");
            if ((lines != null) && (lines.Length > 0))
            {
                if (lines[0] == "1")
                {
                    string[] fields = lines[1].Split('^');
                    if ((fields != null) && (fields.Length > 0))
                    {
                        rlist.Add("success");
                        //rlist.Add(fields[1].ToString());
                    }
                }
                else
                {
                    string[] fields = lines[0].Split('^');
                    if ((fields != null) && (fields.Length > 0))
                    {
                        if (fields[0] == "-1")
                        {
                            rlist.Add("error");
                            rlist.Add(fields[1].ToString());
                        }
                    }
                }
            }
            return rlist;
        }

        internal static List<string> ParseCreateMedResponse(string rawData)
        {
            List<string> rlist = new List<string>();
            if (rawData == null)
            {
                rlist.Add("error");
                rlist.Add("VistA error!");
                return rlist;
            }
            string results = rawData.ToString();
            string[] lines = Regex.Split(results, "\r\n");
            if ((lines != null) && (lines.Length > 0))
            {
                if (lines[0] == "1")
                {
                    string[] fields = lines[1].Split('^');
                    if ((fields != null) && (fields.Length > 0))
                    {
                        rlist.Add("success");
                        //rlist.Add(fields[1].ToString());
                    }
                }
                else
                {
                    string[] fields = lines[0].Split('^');
                    if ((fields != null) && (fields.Length > 0))
                    {
                        if (fields[0] == "-1")
                        {
                            rlist.Add("error");
                            rlist.Add(fields[1].ToString());
                        }
                    }
                }
            }
            return rlist;
        }

        internal static List<string> ParseCreateConsultResponse(string rawData)
        {
            List<string> rlist = new List<string>();
            if (rawData == null)
            {
                rlist.Add("error");
                rlist.Add("VistA error!");
                return rlist;
            }
            string results = rawData.ToString();
            string[] lines = Regex.Split(results, "\r\n");
            if ((lines != null) && (lines.Length > 0))
            {
                if (lines[0] == "1")
                {
                    string[] fields = lines[1].Split('^');
                    if ((fields != null) && (fields.Length > 0))
                    {
                        rlist.Add("success");
                        //rlist.Add(fields[1].ToString());
                    }
                }
                else
                {
                    string[] fields = lines[0].Split('^');
                    if ((fields != null) && (fields.Length > 0))
                    {
                        if (fields[0] == "-1")
                        {
                            rlist.Add("error");
                            rlist.Add(fields[1].ToString());
                        }
                    }
                }
            }
            return rlist;
        }

        internal static List<string> ParseCreateRadResponse(string rawData)
        {
            List<string> rlist = new List<string>();
            if (rawData == null)
            {
                rlist.Add("error");
                rlist.Add("VistA error!");
                return rlist;
            }
            string results = rawData.ToString();
            string[] lines = Regex.Split(results, "\r\n");
            if ((lines != null) && (lines.Length > 0))
            {
                if (lines[0] == "1")
                {
                    string[] fields = lines[1].Split('^');
                    if ((fields != null) && (fields.Length > 0))
                    {
                        rlist.Add("success");
                        //rlist.Add(fields[1].ToString());
                    }
                }
                else
                {
                    string[] fields = lines[0].Split('^');
                    if ((fields != null) && (fields.Length > 0))
                    {
                        if (fields[0] == "-1")
                        {
                            rlist.Add("error");
                            rlist.Add(fields[1].ToString());
                        }
                    }
                }
            }
            return rlist;
        }

        internal static List<string> ParseFetchTableResponse(string rawData)
        {
            List<string> rlist = new List<string>();
            if (rawData == null)
            {
                rlist.Add("error");
                rlist.Add("VistA error!");
                return rlist;
            }
            string results = rawData.ToString();
            string[] lines = Regex.Split(results, "\r\n");
            if ((lines != null) && (lines.Length > 0))
            {
                string[] fields = lines[0].Split('^');
                if (fields[0] == "-1")
                {
                    rlist.Add("error");
                    rlist.Add(fields[1].ToString());
                    return rlist;
                }
                // foreach (string line in lines)
                for (int i = 1; lines[i].Length > 0; i++)
                {
                    rlist.Add(lines[i].ToString());
                }
            }
            return rlist;
        }
    }
}